/*
 * $QNXLicenseC: 
 * Copyright 2012, 2013 QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $
 */

#include "startup.h"
#include <arm/mx6x.h>
#include "board.h"

/* 
 * Note that the '594M PFD' is PLL2's PFD1 which is by default 594MHz for Dual/Quad chips, and
 * 528 MHz for Solo/DualLite chips!
 */
#define MX6X_CCM_CBCMR_GPU3D_SHADER_SEL_594M_PFD	(0x2 << MX6X_CCM_CBCMR_GPU3D_SHADER_CLK_SEL_OFFSET)
#define MX6X_CCM_CBCMR_GPU2D_CORE_SEL_PLL3			(0x1 << MX6X_CCM_CBCMR_GPU2D_CLK_SEL_OFFSET)
#define MX6X_CCM_CBCMR_GPU3D_CORE_SEL_MMDC_CH0		(0x0 << MX6X_CCM_CBCMR_GPU3D_CORE_CLK_SEL_OFFSET)
#define MX6X_CCM_CBCMR_GPU3D_CORE_SEL_594M_PFD		(0x2 << MX6X_CCM_CBCMR_GPU3D_CORE_CLK_SEL_OFFSET)

#define IPU2_HSP_CLK_SEL_MMDC_CH0					(0x0 << MX6X_CCM_CSCDR3_IPU2_HSP_CLK_SEL_OFFSET)
#define IPU2_HSP_CLK_DIV2							(0x1 << MX6X_CCM_CSCDR3_IPU2_HSP_PODF_OFFSET)
#define IPU1_HSP_CLK_SEL_MMDC_CH0					(0x0 << MX6X_CCM_CSCDR3_IPU1_HSP_CLK_SEL_OFFSET)
#define IPU1_HSP_CLK_SEL_540MPFD					(0x3 << MX6X_CCM_CSCDR3_IPU1_HSP_CLK_SEL_OFFSET)
#define IPU1_HSP_CLK_DIV2							(0x1 << MX6X_CCM_CSCDR3_IPU1_HSP_PODF_OFFSET)

/*
 * Gate clocks to unused components.
 */
void mx6x_init_clocks()
{

	out32(MX6X_CCM_BASE + MX6X_CCM_CCGR0, MX6X_CCM_CCGR0_RESET);


    out32(MX6X_CCM_BASE + MX6X_CCM_CCGR1, MX6X_CCM_CCGR1_RESET
#ifdef MX6X_DISABLE_CLOCK_CCGR1
		  & ~( MX6X_DISABLE_CLOCK_CCGR1 )
#endif /* MX6X_DISABLE_CLOCK_CCGR1 */
		  );

    out32(MX6X_CCM_BASE + MX6X_CCM_CCGR2, MX6X_CCM_CCGR2_RESET 
#ifdef MX6X_DISABLE_CLOCK_CCGR2
		  & ~( MX6X_DISABLE_CLOCK_CCGR2 )
#endif /* MX6X_DISABLE_CLOCK_CCGR2 */
		  ); 
    

    out32(MX6X_CCM_BASE + MX6X_CCM_CCGR3, MX6X_CCM_CCGR3_RESET
#ifdef MX6X_DISABLE_CLOCK_CCGR3
		  & ~( MX6X_DISABLE_CLOCK_CCGR3 )
#endif /* MX6X_DISABLE_CLOCK_CCGR3 */
		  );


    out32(MX6X_CCM_BASE + MX6X_CCM_CCGR4, MX6X_CCM_CCGR4_RESET
#ifdef MX6X_DISABLE_CLOCK_CCGR4
		  & ~( MX6X_DISABLE_CLOCK_CCGR4 )
#endif /* MX6X_DISABLE_CLOCK_CCGR4 */		  
		  );
	
    out32(MX6X_CCM_BASE + MX6X_CCM_CCGR5, MX6X_CCM_CCGR5_RESET
#ifdef MX6X_DISABLE_CLOCK_CCGR5
		  & ~( MX6X_DISABLE_CLOCK_CCGR5 )
#endif /* MX6X_DISABLE_CLOCK_CCGR5 */
		  );
	
    out32(MX6X_CCM_BASE + MX6X_CCM_CCGR6, MX6X_CCM_CCGR6_RESET
#ifdef MX6X_DISABLE_CLOCK_CCGR6
		  & ~( MX6X_DISABLE_CLOCK_CCGR6 )
#endif /* MX6X_DISABLE_CLOCK_CCGR6 */		  
		  );
    
    out32(MX6X_CCM_BASE + MX6X_CCM_CCGR7, MX6X_CCM_CCGR7_RESET
#ifdef MX6X_DISABLE_CLOCK_CCGR7
		  & ~( MX6X_DISABLE_CLOCK_CCGR7 )
#endif /* MX6X_DISABLE_CLOCK_CCGR7 */		  
		  );
}


/*
 * Init Vivante 3D GPU, 2D GPU (blitter)
 * Solo / DualLite:
 * 3D Shader Clock: 528 MHz
 * 3D Core Clock: 528 MHz
 *
 * Dual / Quad:
 * 3D Shader Clock: 594 MHz
 * 3D Core Clock:   528 MHz
 */

void mx6x_init_gpu3D(void)
{
    int32_t tmp;
    unsigned int regval = 0;

	disable_gpu_clocks();
    
    /*
     * For i.MX6 Dual/Quad chips the section below will set GPU shader parent clock to 594 MHz PFD clock.
     * For i.MX6 Solo/DualLite chips the same register bits configure the GPU2D core clock, therefore
     * for i.MX6 Solo/DualLite chips the section below sets the GPU2D core clock parent to the 528 MHz PFD.
     * Note that the PFD clock frequencies are different between Solo/DualLite and Dual/Quad.
     */
    tmp = in32(MX6X_CCM_BASE + MX6X_CCM_CBCMR) & ~MX6X_CCM_CBCMR_GPU3D_SHADER_CLK_SEL_MASK;
    tmp |= MX6X_CCM_CBCMR_GPU3D_SHADER_SEL_594M_PFD;
    out32(MX6X_CCM_BASE + MX6X_CCM_CBCMR, tmp);
    
    tmp = in32(MX6X_CCM_BASE + MX6X_CCM_CBCMR) & ~MX6X_CCM_CBCMR_GPU3D_SHADER_PODF_MASK;
    out32(MX6X_CCM_BASE + MX6X_CCM_CBCMR, tmp);

	if (get_mx6_chip_type() == MX6_CHIP_TYPE_DUAL_LITE_OR_SOLO)
	{
		/* set GPU3D core parent clock to 594MHz PFD which is actually 528MHz on Solo/DualLite */
		tmp = in32(MX6X_CCM_BASE + MX6X_CCM_CBCMR) & ~MX6X_CCM_CBCMR_GPU3D_CORE_CLK_SEL_MASK;
		tmp |= MX6X_CCM_CBCMR_GPU3D_CORE_SEL_594M_PFD;
		out32(MX6X_CCM_BASE + MX6X_CCM_CBCMR, tmp);

		// set GPU3D core clock divisor to 1 (bit value 000)
		tmp = in32(MX6X_CCM_BASE + MX6X_CCM_CBCMR) & ~MX6X_CCM_CBCMR_GPU3D_CORE_PODF_MASK;
		out32(MX6X_CCM_BASE + MX6X_CCM_CBCMR, tmp);

		// Note that on Solo/DualLite 2D core clock source is GPU 3D shader clock and is not configurable.

		// GPU3D axi source from mmdc0
		tmp = in32(MX6X_CCM_BASE + MX6X_CCM_CBCMR) & ~MX6X_CCM_CBCMR_GPU3D_AXI_CLK_SEL_MASK;
		out32(MX6X_CCM_BASE + MX6X_CCM_CBCMR, tmp);

		// GPU2D axi source from mmdc0
		tmp = in32(MX6X_CCM_BASE + MX6X_CCM_CBCMR) & ~MX6X_CCM_CBCMR_GPU2D_AXI_CLK_SEL_MASK;
		out32(MX6X_CCM_BASE + MX6X_CCM_CBCMR, tmp);

		// Source IPU clock (ipu1_hsp) from 540M PFD and scale by 2 producing a 270MHz IPU clock 
		tmp = in32(MX6X_CCM_BASE + MX6X_CCM_CSCDR3);
		tmp &= ~(MX6X_CCM_CSCDR3_IPU1_HSP_CLK_SEL_MASK | MX6X_CCM_CSCDR3_IPU1_HSP_PODF_MASK);
		tmp |= (IPU1_HSP_CLK_SEL_540MPFD | IPU1_HSP_CLK_DIV2);
		out32(MX6X_CCM_BASE + MX6X_CCM_CSCDR3, tmp);

	}
	else
	{
		// Set GPU3D core parent clock to multi port DRAM/DDR controller (mmdc).
		tmp = in32(MX6X_CCM_BASE + MX6X_CCM_CBCMR) & ~MX6X_CCM_CBCMR_GPU3D_CORE_CLK_SEL_MASK;
		tmp |= MX6X_CCM_CBCMR_GPU3D_CORE_SEL_MMDC_CH0;
		out32(MX6X_CCM_BASE + MX6X_CCM_CBCMR, tmp);

		// Set GPU3D core clock divisor to 1 (bit value 000)
		tmp = in32(MX6X_CCM_BASE + MX6X_CCM_CBCMR) & ~MX6X_CCM_CBCMR_GPU3D_CORE_PODF_MASK;
		out32(MX6X_CCM_BASE + MX6X_CCM_CBCMR, tmp);

		// Source IPU clocks (ipu1_hsp, ipu2_hsp) from MMDC_CH0 clock and scale by 2 producing a 264MHz IPU clock 
		tmp = in32(MX6X_CCM_BASE + MX6X_CCM_CSCDR3);
		tmp &= ~(MX6X_CCM_CSCDR3_IPU1_HSP_CLK_SEL_MASK | MX6X_CCM_CSCDR3_IPU1_HSP_PODF_MASK |
				MX6X_CCM_CSCDR3_IPU2_HSP_CLK_SEL_MASK | MX6X_CCM_CSCDR3_IPU2_HSP_PODF_MASK);
		tmp |= (IPU1_HSP_CLK_SEL_MMDC_CH0 | IPU1_HSP_CLK_DIV2 | IPU2_HSP_CLK_SEL_MMDC_CH0 |
				IPU2_HSP_CLK_DIV2);
		out32(MX6X_CCM_BASE + MX6X_CCM_CSCDR3, tmp);

		if (get_mx6_chip_type() > MX6_CHIP_REV_1_1)
		{
			tmp = in32(MX6X_CCM_BASE + MX6X_CCM_CBCMR) & 
				~(MX6X_CCM_CBCMR_GPU2D_CORE_PODF_MASK | MX6X_CCM_CBCMR_GPU2D_CLK_SEL_MASK);
			tmp |= MX6X_CCM_CBCMR_GPU2D_CORE_SEL_PLL3;
			out32(MX6X_CCM_BASE + MX6X_CCM_CBCMR, tmp);
		}
	}

	enable_gpu_clocks();

	/*
	 * This clock initialization is only for the rev1.1 CPU
	 * PLL5 clock initialization
	 */
    if(get_mx6_chip_rev() >=  MX6_CHIP_REV_1_1)
    {
	out32(MX6X_ANATOP_BASE + MX6X_ANATOP_PLL5_VIDEO_SET, 0x00012000);

    	// For TO1.1 cpu, select pll5
	out32(MX6X_ANATOP_BASE + MX6X_ANATOP_PLL5_VIDEO_NUM, 50000);
	out32(MX6X_ANATOP_BASE + MX6X_ANATOP_PLL5_VIDEO_DENOM, 100000);
	out32(MX6X_ANATOP_BASE + MX6X_ANATOP_PLL5_VIDEO, 0x00012038);

	regval = in32(MX6X_ANATOP_BASE + MX6X_ANATOP_PLL5_VIDEO);
	while ((!regval) & 0x80000000);

	out32(MX6X_ANATOP_BASE + MX6X_ANATOP_PLL5_VIDEO_CLR, 0x00011000);
    }

}

void mx6_init_usdhc_clk(void)
{
	/* Setting the divider for the esdhc interfaces
	 * to default value which is divide by 2
	 */
	out32(MX6X_CCM_BASE + MX6X_CCM_CSCDR1, MX6X_CCM_CSCDR1_USDHC1_PODF_DIV_2 |
			                               MX6X_CCM_CSCDR1_USDHC2_PODF_DIV_2 |
			                               MX6X_CCM_CSCDR1_USDHC3_PODF_DIV_2 |
			                               MX6X_CCM_CSCDR1_USDHC4_PODF_DIV_2);
}

#if defined(__QNXNTO__) && defined(__USESRCVERSION)
#include <sys/srcversion.h>
__SRCVERSION("$URL: http://svn/product/branches/6.6.0/trunk/hardware/startup/boards/imx6x/mx6x_init_clocks.c $ $Rev: 709644 $")
#endif
