#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <devctl.h>
#include <hw/i2c.h>

#define RTC_I2C_ADDRESS 0x6f

static int twifd;
static int rtc_init(void);
static int rtctemp_read(int twifd);
static void rtc_enabletemp(int twifd);
static int rtc_peek8(int twifd, unsigned short addr);
static int rtc_poke8(int twifd, unsigned short addr, unsigned char value);

static void usage(char **argv) {
	fprintf(stderr, "Usage: %s [OPTION] ...\n"
	  "Technologic Systems RTC tempsensor access\n"
	  "\n"
	  "  -h            This message\n"
	  "  -g            Read the RTC temperature\n",
	  argv[0]
	);
}


int main(int argc, char **argv)
{
   int c;   
	int temp;
	
	if(argc == 1) {
		usage(argv);
		return(1);
	}
	twifd = rtc_init();
	if(twifd == -1)
		return 1;

	while((c = getopt(argc, argv, "hg")) != -1) {
		switch (c) {
		case 'g':
			temp = rtctemp_read(twifd);
			printf("rtctemp_millicelcius=%d\n", temp);
			break;
		case 'h':
		default:
			usage(argv);
		}
	}

	
	close(twifd);
	return 0;
}

static int rtctemp_read(int twifd)
{
	unsigned char data[2];

	data[0] = rtc_peek8(twifd, 0x28);
   data[1] = rtc_peek8(twifd, 0x29);
   
	return ((data[0]|(data[1]<<8))*500)-273000;
}

static void rtc_enabletemp(int twifd)
{	
	unsigned char data;
 
	data = rtc_peek8(twifd, 0x0d);   /* read the BETA register */			
	rtc_poke8(twifd, 0x0d, data | (1 << 7));
	usleep(100000);	
}

static int rtc_init(void)
{
	if(twifd != 0)
		return twifd;

	twifd = open("/dev/i2c1", O_RDWR);
   
	// Enable temp sensor
	rtc_enabletemp(twifd);
	
	return twifd;
}


static int rtc_peek8(int twifd, unsigned short addr)
{
    iov_t           siov[2], riov[2];
    i2c_sendrecv_t  hdr;
    unsigned char reg;
    unsigned char val;

    reg = addr; 
    
    hdr.slave.addr = RTC_I2C_ADDRESS;
    hdr.slave.fmt = I2C_ADDRFMT_7BIT;
    hdr.send_len = 1;
    hdr.recv_len = 1;
    hdr.stop = 1;

    SETIOV(&siov[0], &hdr, sizeof(hdr));
    SETIOV(&siov[1], &reg, sizeof(reg));

    SETIOV(&riov[0], &hdr, sizeof(hdr));
    SETIOV(&riov[1], &val, sizeof(val));

    if (devctlv(twifd, DCMD_I2C_SENDRECV, 2, 2, siov, riov, NULL))
      return -1;
    else
      return val;
}

static int rtc_poke8(int twifd, unsigned short addr, unsigned char value)
{
    iov_t           siov[3];
    i2c_send_t      hdr;
    unsigned char reg;
    
    reg = addr;
    
    hdr.slave.addr = RTC_I2C_ADDRESS;
    hdr.slave.fmt = I2C_ADDRFMT_7BIT;
    hdr.len = 2;   
    hdr.stop = 1;

    SETIOV(&siov[0], &hdr, sizeof(hdr));
    SETIOV(&siov[1], &reg, sizeof(reg));
    SETIOV(&siov[2], &value, sizeof(value));

    return devctlv(twifd, DCMD_I2C_SEND, 3, 0, siov, NULL, NULL);
}

